from __future__ import annotations

from collections.abc import Iterable
from pathlib import Path
from typing import TypedDict


class Source(TypedDict):
    uri: str
    data: str
    mediaType: str


class Event(TypedDict):
    source: Source


def _media_type(path: Path) -> str | None:
    if path.suffix == ".feature":
        return "text/x.cucumber.gherkin+plain"
    if path.name.endswith(".feature.md"):
        return "text/x.cucumber.gherkin+markdown"
    return None


def source_event(path: Path) -> Event:
    event: Event = {
        "source": {
            "uri": str(path),
            "data": path.open(encoding="utf8", newline="").read(),
            "mediaType": _media_type(path),
        },
    }
    return event


class SourceEvents:
    def __init__(self, paths: list[Path]) -> None:
        self.paths = paths

    def enum(self) -> Iterable[Event]:
        return (source_event(path) for path in self.paths)



from __future__ import annotations

from typing import TypedDict

from typing_extensions import NotRequired


class Location(TypedDict):
    line: int
    column: NotRequired[int]


class Comment(TypedDict):
    location: Location
    text: str


class Tag(TypedDict):
    id: str
    location: Location
    name: str


class Cell(TypedDict):
    location: Location
    value: str


class TableRow(TypedDict):
    id: str
    location: Location
    cells: list[Cell]


class DataTable(TypedDict):
    location: Location
    rows: list[TableRow]


class DocString(TypedDict):
    location: Location
    content: str
    delimiter: str
    mediaType: NotRequired[str]


class Step(TypedDict):
    id: str
    location: Location
    keyword: str
    text: str
    keywordType: str
    dataTable: NotRequired[DataTable]
    docString: NotRequired[DocString]


class Background(TypedDict):
    id: str
    location: Location
    keyword: str
    name: str
    description: str
    steps: list[Step]


class Examples(TypedDict):
    id: str
    location: Location
    tags: list[Tag]
    keyword: str
    name: str
    description: str
    tableHeader: TableRow
    tableBody: list[TableRow]


class Scenario(TypedDict):
    id: str
    location: Location
    tags: list[Tag]
    keyword: str
    name: str
    description: str
    steps: list[Step]
    examples: list[Examples]


class Envelope(TypedDict):
    pass


class Rule(TypedDict):
    id: str
    location: Location
    tags: list[Tag]
    keyword: str
    name: str
    description: str
    children: list[Envelope]


class BackgroundEnvelope(Envelope):
    background: Background


class ScenarioEnvelope(Envelope):
    scenario: Scenario


class RuleEnvelope(Envelope):
    rule: Rule


class Feature(TypedDict):
    location: Location
    tags: list[Tag]
    language: str
    keyword: str
    name: str
    description: str
    children: list[Envelope]


class GherkinDocument(TypedDict):
    feature: Feature
    comments: list[Comment]



from __future__ import annotations

import re
from collections.abc import Iterable, Sequence
from typing import Any, TypedDict, cast, overload

from typing_extensions import NotRequired, TypeIs

from gherkin.parser_types import (
    BackgroundEnvelope,
    Cell,
    Envelope,
    GherkinDocument,
    Location,
    Rule,
    RuleEnvelope,
    Scenario,
    ScenarioEnvelope,
    Step,
    Tag,
)
from gherkin.stream.id_generator import IdGenerator


class GherkinDocumentWithURI(GherkinDocument):
    uri: str


class PickleTag(TypedDict):
    astNodeId: str
    name: str


class PickleArgumentEnvelope(TypedDict):
    pass


class PickleArgumentDataTableCell(TypedDict):
    value: str


class PickleArgumentDataTableRow(TypedDict):
    cells: list[PickleArgumentDataTableCell]


class PickleArgumentDataTable(TypedDict):
    rows: list[PickleArgumentDataTableRow]


class PickleArgumentDataTableEnvelope(PickleArgumentEnvelope):
    dataTable: PickleArgumentDataTable


class PickleArgumentDocString(TypedDict):
    content: str | None
    mediaType: NotRequired[str | None]


class PickleArgumentDocStringEnvelope(PickleArgumentEnvelope):
    docString: PickleArgumentDocString


class PickleStep(TypedDict):
    astNodeIds: list[str]
    id: str
    type: str
    text: str
    argument: NotRequired[PickleArgumentEnvelope]


class Pickle(TypedDict):
    astNodeIds: list[str]
    id: str
    location: Location
    tags: list[PickleTag]
    name: str
    language: str
    steps: list[PickleStep]
    uri: str


def is_background_container(container: Envelope) -> TypeIs[BackgroundEnvelope]:
    return "background" in container


def is_rule_container(container: Envelope) -> TypeIs[RuleEnvelope]:
    return "rule" in container


def is_scenario_container(container: Envelope) -> TypeIs[ScenarioEnvelope]:
    return "scenario" in container


class Compiler:
    id_generator: IdGenerator

    def __init__(self, id_generator: IdGenerator | None = None) -> None:
        if id_generator is None:
            id_generator = IdGenerator()
        self.id_generator = id_generator

    def compile(self, gherkin_document: GherkinDocumentWithURI) -> list[Pickle]:
        pickles: list[Pickle] = []
        if "feature" not in gherkin_document:
            return pickles

        feature = gherkin_document["feature"]
        if not feature["children"]:
            return pickles

        uri = gherkin_document["uri"]
        feature_tags = feature["tags"]
        language = feature["language"]
        background_steps: list[Step] = []
        for scenario_definition in feature["children"]:
            if is_background_container(scenario_definition):
                if scenario_definition["background"]["steps"]:
                    background_steps += scenario_definition["background"]["steps"]
            elif is_rule_container(scenario_definition):
                self._compile_rule(
                    uri,
                    feature_tags,
                    background_steps,
                    scenario_definition["rule"],
                    language,
                    pickles,
                )
            else:
                assert is_scenario_container(scenario_definition)
                scenario = scenario_definition["scenario"]
                args = (
                    uri,
                    feature_tags,
                    background_steps,
                    scenario,
                    language,
                    pickles,
                )
                if not scenario["examples"]:
                    self._compile_scenario(*args)
                else:
                    self._compile_scenario_outline(*args)
        return pickles

    def _compile_rule(
        self,
        uri: str,
        feature_tags: list[Tag],
        feature_background_steps: list[Step],
        rule: Rule,
        language: str,
        pickles: list[Pickle],
    ) -> list[Pickle]:
        tags = list(feature_tags) + list(rule["tags"])
        background_steps = []
        background_steps += feature_background_steps
        for scenario_definition in rule["children"]:
            if is_background_container(scenario_definition):
                if scenario_definition["background"]["steps"]:
                    background_steps += scenario_definition["background"]["steps"]
            else:
                assert is_scenario_container(scenario_definition)
                scenario = scenario_definition["scenario"]
                args = (uri, tags, background_steps, scenario, language, pickles)
                if not scenario["examples"]:
                    self._compile_scenario(*args)
                else:
                    self._compile_scenario_outline(*args)
        return pickles

    def _compile_scenario(
        self,
        uri: str,
        inherited_tags: list[Tag],
        background_steps: list[Any],
        scenario: Scenario,
        language: str,
        pickles: list[Pickle],
    ) -> None:
        tags = list(inherited_tags) + list(scenario["tags"])
        last_keyword_type = "Unknown"
        steps = []
        if scenario["steps"]:
            for step in background_steps + scenario["steps"]:
                last_keyword_type = (
                    last_keyword_type
                    if step["keywordType"] == "Conjunction"
                    else step["keywordType"]
                )
                steps.append(self._pickle_step(step, last_keyword_type))

        pickle: Pickle = {
            "astNodeIds": [scenario["id"]],
            "id": self.id_generator.get_next_id(),
            "location": scenario["location"],
            "tags": self._pickle_tags(tags),
            "name": cast(str, scenario["name"]),
            "language": language,
            "steps": steps,
            "uri": uri,
        }
        pickles.append(pickle)

    def _compile_scenario_outline(
        self,
        uri: str,
        inherited_tags: list[Tag],
        background_steps: list[Step],
        scenario: Scenario,
        language: str,
        pickles: list[Pickle],
    ) -> None:
        for examples in scenario["examples"]:
            if "tableHeader" not in examples:
                continue

            variable_cells = examples["tableHeader"]["cells"]

            for values in examples["tableBody"]:
                value_cells = values["cells"]
                tags = (
                    list(inherited_tags)
                    + list(scenario["tags"])
                    + list(examples["tags"])
                )
                last_keyword_type = None
                steps = []
                if scenario["steps"]:
                    for step in background_steps:
                        last_keyword_type = (
                            last_keyword_type
                            if step["keywordType"] == "Conjunction"
                            else step["keywordType"]
                        )
                        steps.append(self._pickle_step(step, last_keyword_type))

                if scenario["steps"]:
                    for outline_step in scenario["steps"]:
                        last_keyword_type = (
                            last_keyword_type
                            if outline_step["keywordType"] == "Conjunction"
                            else outline_step["keywordType"]
                        )
                        step_text = self._interpolate(
                            outline_step["text"],
                            variable_cells,
                            value_cells,
                        )
                        argument = self._create_pickle_arguments(
                            outline_step,
                            variable_cells,
                            value_cells,
                        )
                        _pickle_step: PickleStep = {
                            "astNodeIds": [outline_step["id"], values["id"]],
                            "id": self.id_generator.get_next_id(),
                            "type": last_keyword_type,
                            "text": step_text,
                        }
                        if argument is not None:
                            _pickle_step["argument"] = argument

                        steps.append(_pickle_step)

                pickle: Pickle = {
                    "astNodeIds": [scenario["id"], values["id"]],
                    "id": self.id_generator.get_next_id(),
                    "location": values["location"],
                    "name": self._interpolate(
                        scenario["name"],
                        variable_cells,
                        value_cells,
                    ),
                    "language": language,
                    "steps": steps,
                    "tags": self._pickle_tags(tags),
                    "uri": uri,
                }
                pickles.append(pickle)

    def _create_pickle_arguments(
        self,
        step: Step,
        variables: Sequence[Cell],
        values: Sequence[Cell],
    ) -> PickleArgumentEnvelope | None:
        if "dataTable" in step:
            table: PickleArgumentDataTable = {"rows": []}
            for row in step["dataTable"]["rows"]:
                cells: list[PickleArgumentDataTableCell] = [
                    {"value": self._interpolate(cell["value"], variables, values)}
                    for cell in row["cells"]
                ]
                table["rows"].append({"cells": cells})
            return PickleArgumentDataTableEnvelope(dataTable=table)
        if "docString" in step:
            argument = step["docString"]
            docstring: PickleArgumentDocString = {
                "content": self._interpolate(argument["content"], variables, values),
            }
            if "mediaType" in argument:
                docstring["mediaType"] = self._interpolate(
                    argument["mediaType"],
                    variables,
                    values,
                )
            return PickleArgumentDocStringEnvelope(docString=docstring)
        return None

    @overload
    def _interpolate(
        self,
        name: str,
        variable_cells: Iterable[Cell],
        value_cells: Sequence[Cell],
    ) -> str: ...

    @overload
    def _interpolate(
        self,
        name: None,
        variable_cells: Iterable[Cell],
        value_cells: Sequence[Cell],
    ) -> None: ...

    def _interpolate(
        self,
        name: str | None,
        variable_cells: Iterable[Cell],
        value_cells: Sequence[Cell],
    ) -> str | None:
        if name is None:
            return name

        for n, variable_cell in enumerate(variable_cells):
            value_cell = value_cells[n]
            # For the case of trailing backslash, re-escaping backslashes are needed
            reescaped_value = re.sub(r"\\", r"\\\\", value_cell["value"])
            name = re.sub(f"<{variable_cell['value']}>", reescaped_value, name)
        return name

    def _pickle_step(self, step: Step, keyword_type: str) -> PickleStep:
        pickle_step: PickleStep = {
            "astNodeIds": [step["id"]],
            "id": self.id_generator.get_next_id(),
            "type": keyword_type,
            "text": step["text"],
        }
        argument = self._create_pickle_arguments(step, [], [])
        if argument is not None:
            pickle_step["argument"] = argument
        return pickle_step

    def _pickle_tags(self, tags: Iterable[Tag]) -> list[PickleTag]:
        return [self._pickle_tag(tag) for tag in tags]

    @staticmethod
    def _pickle_tag(tag: Tag) -> PickleTag:
        return {"astNodeId": tag["id"], "name": tag["name"]}



from __future__ import annotations

import typing

if typing.TYPE_CHECKING:
    from gherkin.token import Location, Token


class ParserError(Exception):
    pass


class ParserException(ParserError):
    def __init__(self, message: str, location: Location) -> None:
        self.location = location
        super().__init__(f"({location['line']}:{location.get('column', 0)}): {message}")


class NoSuchLanguageException(ParserException):
    def __init__(self, language: str, location: Location) -> None:
        super().__init__(f"Language not supported: {language}", location)


class AstBuilderException(ParserException):
    pass


class UnexpectedEOFException(ParserException):
    def __init__(
        self,
        received_token: Token,
        expected_token_types: list[str],
        state_comment: str,
    ) -> None:
        message = f"unexpected end of file, expected: {', '.join(expected_token_types)}"
        super().__init__(message, received_token.location)


class UnexpectedTokenException(ParserException):
    def __init__(
        self,
        received_token: Token,
        expected_token_types: list[str],
        state_comment: str,
    ) -> None:
        message = f"expected: {', '.join(expected_token_types)}, got '{received_token.token_value().strip()}'"
        column = received_token.location.get("column", None)
        location = (
            received_token.location
            if column
            else {
                "line": received_token.location["line"],
                "column": received_token.line.indent + 1,
            }
        )
        super().__init__(message, location)


class CompositeParserException(ParserError):
    def __init__(self, errors: list[ParserException]) -> None:
        self.errors = errors
        super().__init__(
            "Parser errors:\n" + "\n".join([error.args[0] for error in errors]),
        )



from __future__ import annotations

from .ast_builder import AstBuilder
from .token import Token


class TokenFormatterBuilder(AstBuilder):
    _tokens: list[Token]

    def __init__(self) -> None:
        self.reset()
        self._tokens = []

    def reset(self) -> None:
        self._tokens = []

    def build(self, token: Token) -> None:
        self._tokens.append(token)

    def start_rule(self, rule_type: str) -> None:
        pass

    def end_rule(self, rule_type: str) -> None:
        pass

    def get_result(self) -> str:
        return "\n".join([self._format_token(token) for token in self._tokens])

    @staticmethod
    def _format_token(token: Token) -> str:
        if token.eof():
            return "EOF"

        return "".join(
            [
                "(",
                str(token.location["line"]),
                ":",
                str(token.location["column"]),
                ")",
                token.matched_type,
                ":",
                (
                    "".join(
                        [
                            "(",
                            (
                                token.matched_keyword_type
                                if token.matched_keyword_type
                                else ""
                            ),
                            ")",
                            token.matched_keyword,
                        ],
                    )
                    if token.matched_keyword
                    else ""
                ),
                "/",
                (token.matched_text if token.matched_text else ""),
                "/",
                ",".join(
                    [
                        str(item["column"]) + ":" + item["text"]
                        for item in token.matched_items
                    ],
                ),
            ],
        )



# This file is generated. Do not edit! Edit gherkin-python.razor instead.
from __future__ import annotations

from collections import deque
from collections.abc import Callable
from typing import TypeVar, cast

from .ast_builder import AstBuilder
from .errors import (
    CompositeParserException,
    ParserException,
    UnexpectedEOFException,
    UnexpectedTokenException,
)
from .parser_types import GherkinDocument
from .token import Token
from .token_matcher import TokenMatcher
from .token_scanner import TokenScanner

_T = TypeVar("_T")
_U = TypeVar("_U")
_V = TypeVar("_V")

RULE_TYPE = [
    "None",
    "_EOF",  # #EOF
    "_Empty",  # #Empty
    "_Comment",  # #Comment
    "_TagLine",  # #TagLine
    "_FeatureLine",  # #FeatureLine
    "_RuleLine",  # #RuleLine
    "_BackgroundLine",  # #BackgroundLine
    "_ScenarioLine",  # #ScenarioLine
    "_ExamplesLine",  # #ExamplesLine
    "_StepLine",  # #StepLine
    "_DocStringSeparator",  # #DocStringSeparator
    "_TableRow",  # #TableRow
    "_Language",  # #Language
    "_Other",  # #Other
    "GherkinDocument",  # GherkinDocument! := Feature?
    "Feature",  # Feature! := FeatureHeader Background? ScenarioDefinition* Rule*
    "FeatureHeader",  # FeatureHeader! := #Language? Tags? #FeatureLine DescriptionHelper
    "Rule",  # Rule! := RuleHeader Background? ScenarioDefinition*
    "RuleHeader",  # RuleHeader! := Tags? #RuleLine DescriptionHelper
    "Background",  # Background! := #BackgroundLine DescriptionHelper Step*
    "ScenarioDefinition",  # ScenarioDefinition! [#Empty|#Comment|#TagLine->#ScenarioLine] := Tags? Scenario
    "Scenario",  # Scenario! := #ScenarioLine DescriptionHelper Step* ExamplesDefinition*
    "ExamplesDefinition",  # ExamplesDefinition! [#Empty|#Comment|#TagLine->#ExamplesLine] := Tags? Examples
    "Examples",  # Examples! := #ExamplesLine DescriptionHelper ExamplesTable?
    "ExamplesTable",  # ExamplesTable! := #TableRow #TableRow*
    "Step",  # Step! := #StepLine StepArg?
    "StepArg",  # StepArg := (DataTable | DocString)
    "DataTable",  # DataTable! := #TableRow+
    "DocString",  # DocString! := #DocStringSeparator #Other* #DocStringSeparator
    "Tags",  # Tags! := #TagLine+
    "DescriptionHelper",  # DescriptionHelper := #Empty* Description?
    "Description",  # Description! := (#Other | #Comment)+
]


class ParserContext:
    def __init__(
        self,
        token_scanner: TokenScanner,
        token_matcher: TokenMatcher,
        token_queue: deque[Token],
        errors: list[ParserException],
    ) -> None:
        self.token_scanner = token_scanner
        self.token_matcher = token_matcher
        self.token_queue = token_queue
        self.errors = errors


class Parser:
    def __init__(self, ast_builder: AstBuilder | None = None) -> None:
        self.ast_builder = ast_builder if ast_builder is not None else AstBuilder()
        self.stop_at_first_error = False

    def parse(
        self,
        token_scanner_or_str: TokenScanner | str,
        token_matcher: TokenMatcher | None = None,
    ) -> GherkinDocument:
        token_scanner = (
            TokenScanner(token_scanner_or_str)
            if isinstance(token_scanner_or_str, str)
            else token_scanner_or_str
        )
        self.ast_builder.reset()
        if token_matcher is None:
            token_matcher = TokenMatcher()
        token_matcher.reset()
        context = ParserContext(token_scanner, token_matcher, deque(), [])

        self.start_rule(context, "GherkinDocument")
        state = 0
        token = None
        while True:
            token = self.read_token(context)
            state = self.match_token(state, token, context)
            if token.eof():
                break

        self.end_rule(context, "GherkinDocument")

        if context.errors:
            raise CompositeParserException(context.errors)

        return cast(GherkinDocument, self.get_result())

    def build(self, context: ParserContext, token: Token) -> None:
        self.handle_ast_error(context, token, self.ast_builder.build)

    def add_error(self, context: ParserContext, error: ParserException) -> None:
        if str(error) not in (str(e) for e in context.errors):
            context.errors.append(error)
            if len(context.errors) > 10:
                raise CompositeParserException(context.errors)

    def start_rule(self, context: ParserContext, rule_type: str) -> None:
        self.handle_ast_error(context, rule_type, self.ast_builder.start_rule)

    def end_rule(self, context: ParserContext, rule_type: str) -> None:
        self.handle_ast_error(context, rule_type, self.ast_builder.end_rule)

    def get_result(self) -> object:
        return self.ast_builder.get_result()

    def read_token(self, context: ParserContext) -> Token:
        if context.token_queue:
            return context.token_queue.popleft()
        return context.token_scanner.read()

    def match_EOF(self, context: ParserContext, token: Token) -> bool:
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_EOF,
        )

    def match_Empty(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_Empty,
        )

    def match_Comment(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_Comment,
        )

    def match_TagLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_TagLine,
        )

    def match_FeatureLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_FeatureLine,
        )

    def match_RuleLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_RuleLine,
        )

    def match_BackgroundLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_BackgroundLine,
        )

    def match_ScenarioLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_ScenarioLine,
        )

    def match_ExamplesLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_ExamplesLine,
        )

    def match_StepLine(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_StepLine,
        )

    def match_DocStringSeparator(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_DocStringSeparator,
        )

    def match_TableRow(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_TableRow,
        )

    def match_Language(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_Language,
        )

    def match_Other(self, context: ParserContext, token: Token) -> bool:
        if token.eof():
            return False
        return self.handle_external_error(
            context,
            False,
            token,
            context.token_matcher.match_Other,
        )

    def match_token(self, state: int, token: Token, context: ParserContext) -> int:
        state_map: dict[int, Callable[[Token, ParserContext], int]] = {
            0: self.match_token_at_0,
            1: self.match_token_at_1,
            2: self.match_token_at_2,
            3: self.match_token_at_3,
            4: self.match_token_at_4,
            5: self.match_token_at_5,
            6: self.match_token_at_6,
            7: self.match_token_at_7,
            8: self.match_token_at_8,
            9: self.match_token_at_9,
            10: self.match_token_at_10,
            11: self.match_token_at_11,
            12: self.match_token_at_12,
            13: self.match_token_at_13,
            14: self.match_token_at_14,
            15: self.match_token_at_15,
            16: self.match_token_at_16,
            17: self.match_token_at_17,
            18: self.match_token_at_18,
            19: self.match_token_at_19,
            20: self.match_token_at_20,
            21: self.match_token_at_21,
            22: self.match_token_at_22,
            23: self.match_token_at_23,
            24: self.match_token_at_24,
            25: self.match_token_at_25,
            26: self.match_token_at_26,
            27: self.match_token_at_27,
            28: self.match_token_at_28,
            29: self.match_token_at_29,
            30: self.match_token_at_30,
            31: self.match_token_at_31,
            32: self.match_token_at_32,
            33: self.match_token_at_33,
            35: self.match_token_at_35,
            36: self.match_token_at_36,
            37: self.match_token_at_37,
            38: self.match_token_at_38,
            39: self.match_token_at_39,
            40: self.match_token_at_40,
            41: self.match_token_at_41,
            42: self.match_token_at_42,
        }

        if state not in state_map:
            raise RuntimeError(f"Unknown state: {state}")

        return state_map[state](token, context)

    # Start
    def match_token_at_0(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.build(context, token)
            return 34
        if self.match_Language(context, token):
            self.start_rule(context, "Feature")
            self.start_rule(context, "FeatureHeader")
            self.build(context, token)
            return 1
        if self.match_TagLine(context, token):
            self.start_rule(context, "Feature")
            self.start_rule(context, "FeatureHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 2
        if self.match_FeatureLine(context, token):
            self.start_rule(context, "Feature")
            self.start_rule(context, "FeatureHeader")
            self.build(context, token)
            return 3
        if self.match_Comment(context, token):
            self.build(context, token)
            return 0
        if self.match_Empty(context, token):
            self.build(context, token)
            return 0

        state_comment = "State: 0 - Start"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Language",
            "#TagLine",
            "#FeatureLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 0

    # GherkinDocument:0>Feature:0>FeatureHeader:0>#Language:0
    def match_token_at_1(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 2
        if self.match_FeatureLine(context, token):
            self.build(context, token)
            return 3
        if self.match_Comment(context, token):
            self.build(context, token)
            return 1
        if self.match_Empty(context, token):
            self.build(context, token)
            return 1

        state_comment = "State: 1 - GherkinDocument:0>Feature:0>FeatureHeader:0>#Language:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#FeatureLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 1

    # GherkinDocument:0>Feature:0>FeatureHeader:1>Tags:0>#TagLine:0
    def match_token_at_2(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.build(context, token)
            return 2
        if self.match_FeatureLine(context, token):
            self.end_rule(context, "Tags")
            self.build(context, token)
            return 3
        if self.match_Comment(context, token):
            self.build(context, token)
            return 2
        if self.match_Empty(context, token):
            self.build(context, token)
            return 2

        state_comment = "State: 2 - GherkinDocument:0>Feature:0>FeatureHeader:1>Tags:0>#TagLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#FeatureLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 2

    # GherkinDocument:0>Feature:0>FeatureHeader:2>#FeatureLine:0
    def match_token_at_3(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "FeatureHeader")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 3
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 4
        if self.match_BackgroundLine(context, token):
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "Background")
            self.build(context, token)
            return 5
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "FeatureHeader")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 4

        state_comment = "State: 3 - GherkinDocument:0>Feature:0>FeatureHeader:2>#FeatureLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#BackgroundLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 3

    # GherkinDocument:0>Feature:0>FeatureHeader:3>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_4(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "FeatureHeader")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 4
        if self.match_BackgroundLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "Background")
            self.build(context, token)
            return 5
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "FeatureHeader")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "FeatureHeader")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 4

        state_comment = "State: 4 - GherkinDocument:0>Feature:0>FeatureHeader:3>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#BackgroundLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 4

    # GherkinDocument:0>Feature:1>Background:0>#BackgroundLine:0
    def match_token_at_5(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Background")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 5
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 6
        if self.match_StepLine(context, token):
            self.start_rule(context, "Step")
            self.build(context, token)
            return 7
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 6

        state_comment = "State: 5 - GherkinDocument:0>Feature:1>Background:0>#BackgroundLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 5

    # GherkinDocument:0>Feature:1>Background:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_6(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 6
        if self.match_StepLine(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 7
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 6

        state_comment = "State: 6 - GherkinDocument:0>Feature:1>Background:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 6

    # GherkinDocument:0>Feature:1>Background:2>Step:0>#StepLine:0
    def match_token_at_7(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.start_rule(context, "DataTable")
            self.build(context, token)
            return 8
        if self.match_DocStringSeparator(context, token):
            self.start_rule(context, "DocString")
            self.build(context, token)
            return 41
        if self.match_StepLine(context, token):
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 7
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Step")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 7
        if self.match_Empty(context, token):
            self.build(context, token)
            return 7

        state_comment = "State: 7 - GherkinDocument:0>Feature:1>Background:2>Step:0>#StepLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#DocStringSeparator",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 7

    # GherkinDocument:0>Feature:1>Background:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0
    def match_token_at_8(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.build(context, token)
            return 8
        if self.match_StepLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 7
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DataTable")
                self.end_rule(context, "Step")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 8
        if self.match_Empty(context, token):
            self.build(context, token)
            return 8

        state_comment = "State: 8 - GherkinDocument:0>Feature:1>Background:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 8

    # GherkinDocument:0>Feature:2>ScenarioDefinition:0>Tags:0>#TagLine:0
    def match_token_at_9(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.build(context, token)
            return 9
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Tags")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_Comment(context, token):
            self.build(context, token)
            return 9
        if self.match_Empty(context, token):
            self.build(context, token)
            return 9

        state_comment = "State: 9 - GherkinDocument:0>Feature:2>ScenarioDefinition:0>Tags:0>#TagLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#ScenarioLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 9

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:0>#ScenarioLine:0
    def match_token_at_10(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 10
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 11
        if self.match_StepLine(context, token):
            self.start_rule(context, "Step")
            self.build(context, token)
            return 12
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 11

        state_comment = "State: 10 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:0>#ScenarioLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 10

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_11(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 11
        if self.match_StepLine(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 12
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Description")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 11

        state_comment = "State: 11 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 11

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:0>#StepLine:0
    def match_token_at_12(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.start_rule(context, "DataTable")
            self.build(context, token)
            return 13
        if self.match_DocStringSeparator(context, token):
            self.start_rule(context, "DocString")
            self.build(context, token)
            return 39
        if self.match_StepLine(context, token):
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 12
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Step")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Step")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Step")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 12
        if self.match_Empty(context, token):
            self.build(context, token)
            return 12

        state_comment = "State: 12 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:0>#StepLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#DocStringSeparator",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 12

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0
    def match_token_at_13(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.build(context, token)
            return 13
        if self.match_StepLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 12
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "DataTable")
                self.end_rule(context, "Step")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DataTable")
                self.end_rule(context, "Step")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 13
        if self.match_Empty(context, token):
            self.build(context, token)
            return 13

        state_comment = "State: 13 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 13

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:0>Tags:0>#TagLine:0
    def match_token_at_14(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.build(context, token)
            return 14
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Tags")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_Comment(context, token):
            self.build(context, token)
            return 14
        if self.match_Empty(context, token):
            self.build(context, token)
            return 14

        state_comment = "State: 14 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:0>Tags:0>#TagLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#ExamplesLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 14

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:0>#ExamplesLine:0
    def match_token_at_15(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 15
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 16
        if self.match_TableRow(context, token):
            self.start_rule(context, "ExamplesTable")
            self.build(context, token)
            return 17
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 16

        state_comment = "State: 15 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:0>#ExamplesLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#TableRow",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 15

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_16(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 16
        if self.match_TableRow(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "ExamplesTable")
            self.build(context, token)
            return 17
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 16

        state_comment = "State: 16 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#TableRow",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 16

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:2>ExamplesTable:0>#TableRow:0
    def match_token_at_17(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.build(context, token)
            return 17
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "ExamplesTable")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "ExamplesTable")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 17
        if self.match_Empty(context, token):
            self.build(context, token)
            return 17

        state_comment = "State: 17 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:2>ExamplesTable:0>#TableRow:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 17

    # GherkinDocument:0>Feature:3>Rule:0>RuleHeader:0>Tags:0>#TagLine:0
    def match_token_at_18(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.build(context, token)
            return 18
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Tags")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 18
        if self.match_Empty(context, token):
            self.build(context, token)
            return 18

        state_comment = "State: 18 - GherkinDocument:0>Feature:3>Rule:0>RuleHeader:0>Tags:0>#TagLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 18

    # GherkinDocument:0>Feature:3>Rule:0>RuleHeader:1>#RuleLine:0
    def match_token_at_19(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "RuleHeader")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 20
        if self.match_BackgroundLine(context, token):
            self.end_rule(context, "RuleHeader")
            self.start_rule(context, "Background")
            self.build(context, token)
            return 21
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "RuleHeader")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "RuleHeader")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "RuleHeader")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "RuleHeader")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 20

        state_comment = "State: 19 - GherkinDocument:0>Feature:3>Rule:0>RuleHeader:1>#RuleLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#BackgroundLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 19

    # GherkinDocument:0>Feature:3>Rule:0>RuleHeader:2>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_20(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "RuleHeader")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 20
        if self.match_BackgroundLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "RuleHeader")
            self.start_rule(context, "Background")
            self.build(context, token)
            return 21
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "RuleHeader")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "RuleHeader")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "RuleHeader")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "RuleHeader")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 20

        state_comment = "State: 20 - GherkinDocument:0>Feature:3>Rule:0>RuleHeader:2>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#BackgroundLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 20

    # GherkinDocument:0>Feature:3>Rule:1>Background:0>#BackgroundLine:0
    def match_token_at_21(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 21
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 22
        if self.match_StepLine(context, token):
            self.start_rule(context, "Step")
            self.build(context, token)
            return 23
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 22

        state_comment = "State: 21 - GherkinDocument:0>Feature:3>Rule:1>Background:0>#BackgroundLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 21

    # GherkinDocument:0>Feature:3>Rule:1>Background:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_22(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 22
        if self.match_StepLine(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 23
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 22

        state_comment = "State: 22 - GherkinDocument:0>Feature:3>Rule:1>Background:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 22

    # GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:0>#StepLine:0
    def match_token_at_23(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.start_rule(context, "DataTable")
            self.build(context, token)
            return 24
        if self.match_DocStringSeparator(context, token):
            self.start_rule(context, "DocString")
            self.build(context, token)
            return 37
        if self.match_StepLine(context, token):
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 23
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Step")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 23
        if self.match_Empty(context, token):
            self.build(context, token)
            return 23

        state_comment = "State: 23 - GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:0>#StepLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#DocStringSeparator",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 23

    # GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0
    def match_token_at_24(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.build(context, token)
            return 24
        if self.match_StepLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 23
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DataTable")
                self.end_rule(context, "Step")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 24
        if self.match_Empty(context, token):
            self.build(context, token)
            return 24

        state_comment = "State: 24 - GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 24

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:0>Tags:0>#TagLine:0
    def match_token_at_25(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.build(context, token)
            return 25
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Tags")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_Comment(context, token):
            self.build(context, token)
            return 25
        if self.match_Empty(context, token):
            self.build(context, token)
            return 25

        state_comment = "State: 25 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:0>Tags:0>#TagLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#ScenarioLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 25

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:0>#ScenarioLine:0
    def match_token_at_26(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 26
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 27
        if self.match_StepLine(context, token):
            self.start_rule(context, "Step")
            self.build(context, token)
            return 28
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 27

        state_comment = "State: 26 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:0>#ScenarioLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 26

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_27(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 27
        if self.match_StepLine(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 28
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Description")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 27

        state_comment = "State: 27 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 27

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:0>#StepLine:0
    def match_token_at_28(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.start_rule(context, "DataTable")
            self.build(context, token)
            return 29
        if self.match_DocStringSeparator(context, token):
            self.start_rule(context, "DocString")
            self.build(context, token)
            return 35
        if self.match_StepLine(context, token):
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 28
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Step")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Step")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Step")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 28
        if self.match_Empty(context, token):
            self.build(context, token)
            return 28

        state_comment = "State: 28 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:0>#StepLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#DocStringSeparator",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 28

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0
    def match_token_at_29(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.build(context, token)
            return 29
        if self.match_StepLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 28
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "DataTable")
                self.end_rule(context, "Step")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DataTable")
                self.end_rule(context, "Step")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DataTable")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 29
        if self.match_Empty(context, token):
            self.build(context, token)
            return 29

        state_comment = "State: 29 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:0>DataTable:0>#TableRow:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 29

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:0>Tags:0>#TagLine:0
    def match_token_at_30(self, token: Token, context: ParserContext) -> int:
        if self.match_TagLine(context, token):
            self.build(context, token)
            return 30
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Tags")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_Comment(context, token):
            self.build(context, token)
            return 30
        if self.match_Empty(context, token):
            self.build(context, token)
            return 30

        state_comment = "State: 30 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:0>Tags:0>#TagLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#TagLine",
            "#ExamplesLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 30

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:0>#ExamplesLine:0
    def match_token_at_31(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Empty(context, token):
            self.build(context, token)
            return 31
        if self.match_Comment(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 32
        if self.match_TableRow(context, token):
            self.start_rule(context, "ExamplesTable")
            self.build(context, token)
            return 33
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.start_rule(context, "Description")
            self.build(context, token)
            return 32

        state_comment = "State: 31 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:0>#ExamplesLine:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Empty",
            "#Comment",
            "#TableRow",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 31

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0
    def match_token_at_32(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_Comment(context, token):
            self.build(context, token)
            return 32
        if self.match_TableRow(context, token):
            self.end_rule(context, "Description")
            self.start_rule(context, "ExamplesTable")
            self.build(context, token)
            return 33
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "Description")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "Description")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Other(context, token):
            self.build(context, token)
            return 32

        state_comment = "State: 32 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:1>DescriptionHelper:1>Description:0>__alt1:0>#Other:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#Comment",
            "#TableRow",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 32

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:2>ExamplesTable:0>#TableRow:0
    def match_token_at_33(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_TableRow(context, token):
            self.build(context, token)
            return 33
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "ExamplesTable")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "ExamplesTable")
                self.end_rule(context, "Examples")
                self.end_rule(context, "ExamplesDefinition")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "ExamplesTable")
            self.end_rule(context, "Examples")
            self.end_rule(context, "ExamplesDefinition")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 33
        if self.match_Empty(context, token):
            self.build(context, token)
            return 33

        state_comment = "State: 33 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:3>ExamplesDefinition:1>Examples:2>ExamplesTable:0>#TableRow:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#TableRow",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 33

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0
    def match_token_at_35(self, token: Token, context: ParserContext) -> int:
        if self.match_DocStringSeparator(context, token):
            self.build(context, token)
            return 36
        if self.match_Other(context, token):
            self.build(context, token)
            return 35

        state_comment = "State: 35 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#DocStringSeparator",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 35

    # GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0
    def match_token_at_36(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_StepLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 28
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "DocString")
                self.end_rule(context, "Step")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 30
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DocString")
                self.end_rule(context, "Step")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 31
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 36
        if self.match_Empty(context, token):
            self.build(context, token)
            return 36

        state_comment = "State: 36 - GherkinDocument:0>Feature:3>Rule:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 36

    # GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0
    def match_token_at_37(self, token: Token, context: ParserContext) -> int:
        if self.match_DocStringSeparator(context, token):
            self.build(context, token)
            return 38
        if self.match_Other(context, token):
            self.build(context, token)
            return 37

        state_comment = "State: 37 - GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#DocStringSeparator",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 37

    # GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0
    def match_token_at_38(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_StepLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 23
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DocString")
                self.end_rule(context, "Step")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 25
        if self.match_TagLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 26
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Rule")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 38
        if self.match_Empty(context, token):
            self.build(context, token)
            return 38

        state_comment = "State: 38 - GherkinDocument:0>Feature:3>Rule:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 38

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0
    def match_token_at_39(self, token: Token, context: ParserContext) -> int:
        if self.match_DocStringSeparator(context, token):
            self.build(context, token)
            return 40
        if self.match_Other(context, token):
            self.build(context, token)
            return 39

        state_comment = "State: 39 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#DocStringSeparator",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 39

    # GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0
    def match_token_at_40(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_StepLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 12
        if self.match_TagLine(context, token):
            if self.lookahead_1(context, token):
                self.end_rule(context, "DocString")
                self.end_rule(context, "Step")
                self.start_rule(context, "ExamplesDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 14
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DocString")
                self.end_rule(context, "Step")
                self.end_rule(context, "Scenario")
                self.end_rule(context, "ScenarioDefinition")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ExamplesLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.start_rule(context, "ExamplesDefinition")
            self.start_rule(context, "Examples")
            self.build(context, token)
            return 15
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Scenario")
            self.end_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 40
        if self.match_Empty(context, token):
            self.build(context, token)
            return 40

        state_comment = "State: 40 - GherkinDocument:0>Feature:2>ScenarioDefinition:1>Scenario:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#StepLine",
            "#TagLine",
            "#ExamplesLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 40

    # GherkinDocument:0>Feature:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0
    def match_token_at_41(self, token: Token, context: ParserContext) -> int:
        if self.match_DocStringSeparator(context, token):
            self.build(context, token)
            return 42
        if self.match_Other(context, token):
            self.build(context, token)
            return 41

        state_comment = "State: 41 - GherkinDocument:0>Feature:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:0>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#DocStringSeparator",
            "#Other",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 41

    # GherkinDocument:0>Feature:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0
    def match_token_at_42(self, token: Token, context: ParserContext) -> int:
        if self.match_EOF(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.end_rule(context, "Feature")
            self.build(context, token)
            return 34
        if self.match_StepLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.start_rule(context, "Step")
            self.build(context, token)
            return 7
        if self.match_TagLine(context, token):
            if self.lookahead_0(context, token):
                self.end_rule(context, "DocString")
                self.end_rule(context, "Step")
                self.end_rule(context, "Background")
                self.start_rule(context, "ScenarioDefinition")
                self.start_rule(context, "Tags")
                self.build(context, token)
                return 9
        if self.match_TagLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.start_rule(context, "Tags")
            self.build(context, token)
            return 18
        if self.match_ScenarioLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "ScenarioDefinition")
            self.start_rule(context, "Scenario")
            self.build(context, token)
            return 10
        if self.match_RuleLine(context, token):
            self.end_rule(context, "DocString")
            self.end_rule(context, "Step")
            self.end_rule(context, "Background")
            self.start_rule(context, "Rule")
            self.start_rule(context, "RuleHeader")
            self.build(context, token)
            return 19
        if self.match_Comment(context, token):
            self.build(context, token)
            return 42
        if self.match_Empty(context, token):
            self.build(context, token)
            return 42

        state_comment = "State: 42 - GherkinDocument:0>Feature:1>Background:2>Step:1>StepArg:0>__alt0:1>DocString:2>#DocStringSeparator:0"  # fmt: skip
        token.detach()
        expected_tokens = [
            "#EOF",
            "#StepLine",
            "#TagLine",
            "#ScenarioLine",
            "#RuleLine",
            "#Comment",
            "#Empty",
        ]
        error = (
            UnexpectedEOFException(token, expected_tokens, state_comment)
            if token.eof()
            else UnexpectedTokenException(token, expected_tokens, state_comment)
        )
        if self.stop_at_first_error:
            raise error
        self.add_error(context, error)
        return 42

    def lookahead_0(self, context: ParserContext, currentToken: Token) -> bool:
        currentToken.detach()
        token = None
        queue = []
        match = False
        # Effectively do-while
        continue_lookahead = True
        while continue_lookahead:
            token = self.read_token(context)
            token.detach()
            queue.append(token)

            if self.match_ScenarioLine(context, token):
                match = True
                break

            continue_lookahead = False

            if self.match_Empty(context, token):
                continue_lookahead = True
                continue
            if self.match_Comment(context, token):
                continue_lookahead = True
                continue
            if self.match_TagLine(context, token):
                continue_lookahead = True
                continue

        context.token_queue.extend(queue)

        return match

    def lookahead_1(self, context: ParserContext, currentToken: Token) -> bool:
        currentToken.detach()
        token = None
        queue = []
        match = False
        # Effectively do-while
        continue_lookahead = True
        while continue_lookahead:
            token = self.read_token(context)
            token.detach()
            queue.append(token)

            if self.match_ExamplesLine(context, token):
                match = True
                break

            continue_lookahead = False

            if self.match_Empty(context, token):
                continue_lookahead = True
                continue
            if self.match_Comment(context, token):
                continue_lookahead = True
                continue
            if self.match_TagLine(context, token):
                continue_lookahead = True
                continue

        context.token_queue.extend(queue)

        return match

    # private

    def handle_ast_error(
        self,
        context: ParserContext,
        argument: _T,
        action: Callable[[_T], object],
    ) -> None:
        self.handle_external_error(context, True, argument, action)

    def handle_external_error(
        self,
        context: ParserContext,
        default_value: _U,
        argument: _T,
        action: Callable[[_T], _V],
    ) -> _V | _U:
        if self.stop_at_first_error:
            return action(argument)

        try:
            return action(argument)
        except CompositeParserException as e:
            for error in e.errors:
                self.add_error(context, error)
        except ParserException as e:
            self.add_error(context, e)
        return default_value



from __future__ import annotations

from collections.abc import Generator, Iterable
from dataclasses import dataclass
from typing import TypedDict

from gherkin.ast_builder import AstBuilder
from gherkin.errors import CompositeParserException, ParserError, ParserException
from gherkin.parser import Parser
from gherkin.pickles.compiler import Compiler, GherkinDocumentWithURI, Pickle
from gherkin.stream.id_generator import IdGenerator
from gherkin.stream.source_events import Event
from gherkin.token import Location
from gherkin.token_matcher import TokenMatcher
from gherkin.token_matcher_markdown import GherkinInMarkdownTokenMatcher


class Source(TypedDict):
    uri: str
    location: Location


class ParseError(TypedDict):
    source: Source
    message: str


class Error(TypedDict):
    parseError: ParseError


def create_errors(errors: Iterable[ParserException], uri: str) -> Generator[Error]:
    for error in errors:
        yield {
            "parseError": {
                "source": {"uri": uri, "location": error.location},
                "message": str(error),
            },
        }


class GherkinDocumentEnvelope(TypedDict):
    gherkinDocument: GherkinDocumentWithURI


class PickleEnvelope(TypedDict):
    pickle: Pickle


class GherkinEvents:
    @dataclass
    class Options:
        print_source: bool
        print_ast: bool
        print_pickles: bool

    def __init__(self, options: Options) -> None:
        self.options = options
        self.id_generator = IdGenerator()
        self.parser = Parser(ast_builder=AstBuilder(self.id_generator))
        self.compiler = Compiler(self.id_generator)

    def enum(
        self,
        source_event: Event,
    ) -> Generator[Event | Error | GherkinDocumentEnvelope | PickleEnvelope,]:
        uri = source_event["source"]["uri"]
        source = source_event["source"]["data"]

        try:
            matcher = None
            if source_event["source"]["mediaType"] == "text/x.cucumber.gherkin+plain":
                matcher = TokenMatcher()
            elif (
                source_event["source"]["mediaType"]
                == "text/x.cucumber.gherkin+markdown"
            ):
                matcher = GherkinInMarkdownTokenMatcher()
            gherkin_document = self.parser.parse(source, matcher)
            gherkin_document_with_uri: GherkinDocumentWithURI = {
                **gherkin_document,
                "uri": uri,
            }

            if self.options.print_source:
                yield source_event

            if self.options.print_ast:
                yield {"gherkinDocument": gherkin_document_with_uri}

            if self.options.print_pickles:
                pickles = self.compiler.compile(gherkin_document_with_uri)
                for pickle in pickles:
                    yield {"pickle": pickle}
        except CompositeParserException as e:
            yield from create_errors(e.errors, uri)
        except ParserError as e:
            yield from create_errors([e], uri)



from __future__ import annotations

import re
import textwrap
from collections import defaultdict
from collections.abc import Iterable
from typing import TypedDict

from .dialect import Dialect
from .errors import NoSuchLanguageException
from .parser_types import Location
from .token import Token


def indent(text: str, amount: int, ch: str = " ") -> str:
    return textwrap.indent(text, amount * ch)


class MatchedItems(TypedDict):
    column: int
    text: str


class TokenMatcher:
    LANGUAGE_RE = re.compile(r"^\s*#\s*language\s*:\s*([a-zA-Z\-_]+)\s*$")

    _indent_to_remove: int
    _active_doc_string_separator: str | None

    def __init__(self, dialect_name: str = "en") -> None:
        self._default_dialect_name = dialect_name
        self._change_dialect(dialect_name)
        self.reset()

    def reset(self) -> None:
        if self.dialect_name != self._default_dialect_name:
            self._change_dialect(self._default_dialect_name)
        self._indent_to_remove = 0
        self._active_doc_string_separator = None

    def match_FeatureLine(self, token: Token) -> bool:
        return self._match_title_line(
            token,
            "FeatureLine",
            self.dialect.feature_keywords,
        )

    def match_RuleLine(self, token: Token) -> bool:
        return self._match_title_line(token, "RuleLine", self.dialect.rule_keywords)

    def match_ScenarioLine(self, token: Token) -> bool:
        return self._match_title_line(
            token,
            "ScenarioLine",
            self.dialect.scenario_keywords,
        ) or self._match_title_line(
            token,
            "ScenarioLine",
            self.dialect.scenario_outline_keywords,
        )

    def match_BackgroundLine(self, token: Token) -> bool:
        return self._match_title_line(
            token,
            "BackgroundLine",
            self.dialect.background_keywords,
        )

    def match_ExamplesLine(self, token: Token) -> bool:
        return self._match_title_line(
            token,
            "ExamplesLine",
            self.dialect.examples_keywords,
        )

    def match_TableRow(self, token: Token) -> bool:
        if not token.line.startswith("|"):
            return False
        # TODO: indent
        self._set_token_matched(token, "TableRow", items=token.line.table_cells)
        return True

    def match_StepLine(self, token: Token) -> bool:
        keywords = self._sorted_step_keywords
        for keyword in (k for k in keywords if token.line.startswith(k)):
            title = token.line.get_rest_trimmed(len(keyword))
            keyword_types = self.keyword_types[keyword]
            keyword_type = keyword_types[0] if len(keyword_types) == 1 else "Unknown"
            self._set_token_matched(
                token,
                "StepLine",
                title,
                keyword,
                keyword_type=keyword_type,
            )
            return True

        return False

    def match_Comment(self, token: Token) -> bool:
        if not token.line.startswith("#"):
            return False

        text = token.line._line_text  # take the entire line, including leading space
        self._set_token_matched(token, "Comment", text, indent=0)
        return True

    def match_Empty(self, token: Token) -> bool:
        if not token.line.is_empty():
            return False

        self._set_token_matched(token, "Empty", indent=0)
        return True

    def match_Language(self, token: Token) -> bool:
        match = self.LANGUAGE_RE.match(token.line.get_line_text())
        if not match:
            return False

        dialect_name = match.group(1)
        self._set_token_matched(token, "Language", dialect_name)
        self._change_dialect(dialect_name, token.location)
        return True

    def match_TagLine(self, token: Token) -> bool:
        if not token.line.startswith("@"):
            return False

        self._set_token_matched(token, "TagLine", items=token.line.tags)
        return True

    def match_DocStringSeparator(self, token: Token) -> bool:
        if not self._active_doc_string_separator:
            # open
            return self._match_DocStringSeparator(
                token,
                '"""',
                True,
            ) or self._match_DocStringSeparator(token, "```", True)
        # close
        return self._match_DocStringSeparator(
            token,
            self._active_doc_string_separator,
            False,
        )

    @staticmethod
    def _default_docstring_content_type() -> None:
        return None

    def _match_DocStringSeparator(
        self,
        token: Token,
        separator: str,
        is_open: bool,
    ) -> bool:
        if not token.line.startswith(separator):
            return False

        content_type = self._default_docstring_content_type()
        if is_open:
            content_type = token.line.get_rest_trimmed(len(separator))
            self._active_doc_string_separator = separator
            self._indent_to_remove = token.line.indent
        else:
            self._active_doc_string_separator = None
            self._indent_to_remove = 0

        # TODO: Use the separator as keyword. That's needed for pretty printing.
        self._set_token_matched(token, "DocStringSeparator", content_type, separator)
        return True

    def match_Other(self, token: Token) -> bool:
        # take the entire line, except removing DocString indents
        text = token.line.get_line_text(self._indent_to_remove)
        self._set_token_matched(
            token,
            "Other",
            self._unescaped_docstring(text),
            indent=0,
        )
        return True

    def match_EOF(self, token: Token) -> bool:
        if not token.eof():
            return False

        self._set_token_matched(token, "EOF")
        return True

    def _match_title_line(
        self,
        token: Token,
        token_type: str,
        keywords: Iterable[str],
    ) -> bool:
        for keyword in (k for k in keywords if token.line.startswith_title_keyword(k)):
            title = token.line.get_rest_trimmed(len(keyword) + len(":"))
            self._set_token_matched(token, token_type, title, keyword)
            return True

        return False

    def _set_token_matched(
        self,
        token: Token,
        matched_type: str | None,
        text: str | None = None,
        keyword: str | None = None,
        keyword_type: str | None = None,
        indent: int | None = None,
        items: list[MatchedItems] | None = None,
    ) -> None:
        if items is None:
            items = []
        token.matched_type = matched_type
        # text == '' should not result in None
        token.matched_text = text.rstrip("\r\n") if text is not None else None
        token.matched_keyword = keyword
        token.matched_keyword_type = keyword_type
        if indent is not None:
            token.matched_indent = indent
        else:
            token.matched_indent = token.line.indent if token.line else 0
        token.matched_items = items
        token.location["column"] = token.matched_indent + 1
        token.matched_gherkin_dialect = self.dialect_name

    def _change_dialect(
        self,
        dialect_name: str,
        location: Location | None = None,
    ) -> None:
        dialect = Dialect.for_name(dialect_name)
        if not dialect:
            raise NoSuchLanguageException(dialect_name, location)

        self.dialect_name = dialect_name
        self.dialect = dialect
        self.keyword_types = defaultdict(list)
        for keyword in self.dialect.given_keywords:
            self.keyword_types[keyword].append("Context")
        for keyword in self.dialect.when_keywords:
            self.keyword_types[keyword].append("Action")
        for keyword in self.dialect.then_keywords:
            self.keyword_types[keyword].append("Outcome")
        for keyword in self.dialect.and_keywords + self.dialect.but_keywords:
            self.keyword_types[keyword].append("Conjunction")

        self._sorted_step_keywords = sorted(
            (
                self.dialect.given_keywords
                + self.dialect.when_keywords
                + self.dialect.then_keywords
                + self.dialect.and_keywords
                + self.dialect.but_keywords
            ),
            key=len,
            reverse=True,
        )

    def _unescaped_docstring(self, text: str) -> str:
        if self._active_doc_string_separator == '"""':
            return text.replace('\\"\\"\\"', '"""')
        if self._active_doc_string_separator == "```":
            return text.replace("\\`\\`\\`", "```")
        return text
